from openmmtools import alchemy
from openmm import app, unit
from openmmtools import states, mcmc, multistate
import copy
import netCDF4 as nc

reportInterval = 1000
crd = app.AmberInpcrdFile(f"ethanol.rst7")
top = app.AmberPrmtopFile(f"ethanol.prm7")

positions = crd.positions
mm_system = top.createSystem(
    nonbondedMethod=app.PME, nonbondedCutoff=1 * unit.nanometer, constraints=app.HBonds
)

alchemical_region = alchemy.AlchemicalRegion(alchemical_atoms=list(range(9)))
factory = alchemy.AbsoluteAlchemicalFactory()
alchemical_system = factory.create_alchemical_system(mm_system, alchemical_region)


thermo_state = states.ThermodynamicState(
    system=alchemical_system,
    temperature=298.0 * unit.kelvin,
    pressure=1.0 * unit.atmosphere,
)
alchemical_state = alchemy.AlchemicalState.from_system(alchemical_system)
compound_state = states.CompoundThermodynamicState(
    thermodynamic_state=thermo_state, composable_states=[alchemical_state]
)

compound_thermodynamic_states = []
electrostatics = [1.0, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1]
sterics = [
    1.0,
    0.95,
    0.9,
    0.85,
    0.8,
    0.75,
    0.7,
    0.65,
    0.6,
    0.55,
    0.5,
    0.4,
    0.3,
    0.2,
    0.1,
    0.0,
]
lambda_list = list(
    zip(
        [
            *electrostatics,
            *[0.0] * len(sterics),
        ],
        [*[1.0] * len(electrostatics), *sterics],
    )
)
# lambda_list = [(1.0, 1.0)]

for electrostatic, steric in lambda_list:
    lambda_state = copy.deepcopy(compound_state)
    lambda_state.lambda_electrostatics = electrostatic
    lambda_state.lambda_sterics = steric
    compound_thermodynamic_states.append(lambda_state)

move = mcmc.LangevinDynamicsMove(
    timestep=2.0 * unit.femtosecond,
    collision_rate=1.0 / unit.picoseconds,
    n_steps=reportInterval,
)
for i in range(len(compound_thermodynamic_states)):
    simulation = multistate.MultiStateSampler(
        mcmc_moves=move, number_of_iterations=100, online_analysis_interval=1000
    )

    storage_path = f"lambda_{i}.nc"
    reporter = multistate.MultiStateReporter(
        storage_path,
        checkpoint_interval=1000,
    )
    simulation.create(
        thermodynamic_states=compound_thermodynamic_states,
        sampler_states=states.SamplerState(
            crd.positions, box_vectors=crd.getBoxVectors()
        ),
        storage=reporter,
        initial_thermodynamic_states=[i],
    )
    simulation.run()
    break
